"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var postcss_1 = __importDefault(require("postcss"));
var applyGlobals_1 = require("../applyGlobals");
test("Handles null/undefined", function () {
    expect(applyGlobals_1.parseGlobals(null)).toEqual({
        vars: new Map(),
        conditions: [],
        globalVarsOnly: false
    });
    expect(applyGlobals_1.parseGlobals(undefined)).toEqual({
        vars: new Map(),
        conditions: [],
        globalVarsOnly: false
    });
});
test("Handles objects", function () {
    expect(applyGlobals_1.parseGlobals({ color: "red" })).toEqual({
        vars: new Map([["color", new Set([0])]]),
        conditions: [{ query: undefined, values: new Map([["color", "red"]]) }],
        globalVarsOnly: false
    });
    expect(applyGlobals_1.parseGlobals({ color: "red", other: "green" })).toEqual({
        vars: new Map([
            ["color", new Set([0])],
            ["other", new Set([0])]
        ]),
        conditions: [
            {
                query: undefined,
                values: new Map([
                    ["color", "red"],
                    ["other", "green"]
                ])
            }
        ],
        globalVarsOnly: false
    });
});
test("Handles basic strings", function () {
    expect(applyGlobals_1.parseGlobals("\n      --color: red;\n    ")).toEqual({
        vars: new Map([["color", new Set([0])]]),
        conditions: [{ query: undefined, values: new Map([["color", "red"]]) }],
        globalVarsOnly: false
    });
    expect(applyGlobals_1.parseGlobals("\n      --color: red;\n      --other: green;\n    ")).toEqual({
        vars: new Map([
            ["color", new Set([0])],
            ["other", new Set([0])]
        ]),
        conditions: [
            {
                query: undefined,
                values: new Map([
                    ["color", "red"],
                    ["other", "green"]
                ])
            }
        ],
        globalVarsOnly: false
    });
});
test("Handles strings with media queries", function () {
    expect(applyGlobals_1.parseGlobals("\n      --color: red;\n\n      @media (prefers-color-scheme: dark) {\n        --color: green;\n      }\n    ")).toEqual({
        vars: new Map([["color", new Set([0, 1])]]),
        conditions: [
            { query: undefined, values: new Map([["color", "red"]]) },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([["color", "green"]])
            }
        ],
        globalVarsOnly: false
    });
    expect(applyGlobals_1.parseGlobals("\n      --color: red;\n      --other: green;\n      \n      @media (prefers-color-scheme: dark) {\n        --color: green;\n        --other: red;\n      }\n    ")).toEqual({
        vars: new Map([
            ["color", new Set([0, 1])],
            ["other", new Set([0, 1])]
        ]),
        conditions: [
            {
                query: undefined,
                values: new Map([
                    ["color", "red"],
                    ["other", "green"]
                ])
            },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([
                    ["color", "green"],
                    ["other", "red"]
                ])
            }
        ],
        globalVarsOnly: false
    });
    expect(applyGlobals_1.parseGlobals("\n      --color: red;\n      --other: green;\n      \n      @media (prefers-color-scheme: dark) {\n        --color: green;\n      }\n    ")).toEqual({
        vars: new Map([
            ["color", new Set([0, 1])],
            ["other", new Set([0])]
        ]),
        conditions: [
            {
                query: undefined,
                values: new Map([
                    ["color", "red"],
                    ["other", "green"]
                ])
            },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([["color", "green"]])
            }
        ],
        globalVarsOnly: false
    });
});
test("Applying non-conditional globals", function () {
    var globals = {
        vars: new Map([["color", new Set([0])]]),
        conditions: [{ query: undefined, values: new Map([["color", "red"]]) }],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      color: var(--color);\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          color: red;\n        }\n      \"\n  ");
});
test("Applying non-conditional globals with other values", function () {
    var globals = {
        vars: new Map([["color", new Set([0])]]),
        conditions: [{ query: undefined, values: new Map([["color", "red"]]) }],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      background: url(some-url) var(--color);\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          background: url(some-url) red;\n        }\n      \"\n  ");
});
test("Applying non-conditional globals with multiple rules", function () {
    var globals = {
        vars: new Map([["color", new Set([0])]]),
        conditions: [{ query: undefined, values: new Map([["color", "red"]]) }],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      background: url(some-url) var(--color);\n    }\n\n    .someOtherRule {\n      background: url(some-url) var(--color);\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          background: url(some-url) red;\n        }\n\n        .someOtherRule {\n          background: url(some-url) red;\n        }\n      \"\n  ");
});
test("Applying conditional globals", function () {
    var globals = {
        vars: new Map([["color", new Set([0, 1])]]),
        conditions: [
            { query: undefined, values: new Map([["color", "red"]]) },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([["color", "green"]])
            }
        ],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      color: var(--color);\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          color: red;\n        }\n    @media (prefers-color-scheme: dark) {\n        .someRule {\n          color: green;\n        }\n    }\n      \"\n  ");
});
test("Applying conditional globals with other non-var decls", function () {
    var globals = {
        vars: new Map([["color", new Set([0, 1])]]),
        conditions: [
            { query: undefined, values: new Map([["color", "red"]]) },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([["color", "green"]])
            }
        ],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      notVar: something;\n      color: var(--color);\n      alsoNotVar: something else;\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          notVar: something;\n          color: red;\n          alsoNotVar: something else;\n        }\n    @media (prefers-color-scheme: dark) {\n        .someRule {\n          notVar: something;\n          color: green;\n          alsoNotVar: something else;\n        }\n    }\n      \"\n  ");
});
test("Applying conditional globals inside media queries", function () {
    var globals = {
        vars: new Map([["color", new Set([0, 1])]]),
        conditions: [
            { query: undefined, values: new Map([["color", "red"]]) },
            {
                query: "(prefers-color-scheme: dark)",
                values: new Map([["color", "green"]])
            }
        ],
        globalVarsOnly: false
    };
    var root = postcss_1["default"].parse("\n    .someRule {\n      color: var(--color);\n    }\n\n    @media (platform: ios) {\n      .someRule {\n        background: var(--color);\n        color: orange;\n      }\n    }\n\n    @media (platform: ios), (platform: android) {\n      .someRule {\n        border-color: var(--color);\n      }\n    }\n  ");
    applyGlobals_1.applyGlobals(root, globals);
    expect(root.toString()).toMatchInlineSnapshot("\n    \"\n        .someRule {\n          color: red;\n        }\n    @media (prefers-color-scheme: dark) {\n        .someRule {\n          color: green;\n        }\n    }\n\n        @media (platform: ios) {\n          .someRule {\n            background: red;\n            color: orange;\n          }\n        }\n\n        @media (platform: ios) and (prefers-color-scheme: dark) {\n          .someRule {\n            background: green;\n            color: orange;\n          }\n    }\n\n        @media (platform: ios), (platform: android) {\n          .someRule {\n            border-color: red;\n          }\n        }\n\n        @media (platform: ios) and (prefers-color-scheme: dark), (platform: android) and (prefers-color-scheme: dark) {\n          .someRule {\n            border-color: green;\n          }\n    }\n      \"\n  ");
});
