"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var cssRegExp_1 = require("../../runtime/cssRegExp");
var getRoot_1 = __importDefault(require("./getRoot"));
var applyGlobals_1 = __importDefault(require("./applyGlobals"));
var util_1 = require("./util");
var types_1 = require("./types");
var walkToArray = function (walker) {
    var nodes = [];
    walker(function (node) { return nodes.push(node); });
    return nodes;
};
var getStyleTuples = function (nodes) {
    return nodes
        .filter(function (node) { return node.type === "decl"; })
        .map(function (node) { return [node.prop, node.value]; });
};
var getStyleTuplesImportedVariables = function (styleTuples) {
    var importedVariablesSet = new Set();
    styleTuples.forEach(function (tuple) {
        var referencedVariableMatches = tuple[1].match(cssRegExp_1.varRegExp);
        if (referencedVariableMatches == null)
            return;
        referencedVariableMatches.forEach(function (match) {
            importedVariablesSet.add(match.match(cssRegExp_1.varRegExpNonGlobal)[1]);
        });
    });
    var importedVariables = Array.from(importedVariablesSet);
    return importedVariables;
};
var transitionAttributes = {
    transition: "_",
    "transition-delay": "delay",
    "transition-duration": "duration",
    "transition-property": "property",
    "transition-timing-function": "timingFunction"
};
var animationAttributes = {
    animation: "_",
    "animation-delay": "delay",
    "animation-duration": "duration",
    "animation-name": "name",
    "animation-timing-function": "timingFunction",
    "animation-iteration-count": "iterations"
};
var getTransitionAnimationParts = function (styleTuples, attributes) {
    var accum = {};
    styleTuples.forEach(function (_a) {
        var _b;
        var key = _a[0], value = _a[1];
        var attributeKey = attributes[key];
        if (attributeKey === "_") {
            accum = (_b = {}, _b["_"] = value, _b);
        }
        else if (attributeKey !== undefined) {
            accum[attributeKey] = value;
        }
        else {
            throw new Error("Missing key");
        }
    });
    return accum;
};
var getCondition = function (rule) {
    var selector = rule.selector;
    var mediaQuery = null;
    if (rule.parent &&
        rule.parent.type === "atrule" &&
        rule.parent.name === "media") {
        mediaQuery = rule.parent.params;
    }
    return selector !== "&" || mediaQuery ? { selector: selector, mediaQuery: mediaQuery } : null;
};
var filteredStyleTuples = function (declarationRules, fn) {
    return declarationRules
        .map(function (rule) {
        var styleTuples = getStyleTuples(rule.nodes).filter(function (styleTuple) {
            return fn(styleTuple[0]);
        });
        return { condition: getCondition(rule), styleTuples: styleTuples };
    })
        .filter(function (style) { return style.styleTuples.length > 0; });
};
var getStyleTuplesMixinsForRule = function (rule) {
    var condition = getCondition(rule);
    var groups = [];
    var currentStyleTuples = null;
    rule.nodes.forEach(function (node) {
        switch (node.type) {
            case "decl": {
                var prop = node.prop, value = node.value;
                if (prop in animationAttributes ||
                    prop in transitionAttributes ||
                    prop.startsWith("--")) {
                    return;
                }
                if (currentStyleTuples == null) {
                    currentStyleTuples = {
                        type: types_1.StyleType.Tuples,
                        condition: condition,
                        styleTuples: [],
                        // Set imported variables after
                        importedVariables: null
                    };
                    groups.push(currentStyleTuples);
                }
                currentStyleTuples.styleTuples.push([prop, value]);
                break;
            }
            case "atrule": {
                if (node.name !== "include") {
                    return;
                }
                currentStyleTuples = null;
                groups.push({
                    type: types_1.StyleType.Mixin,
                    condition: condition,
                    substitution: node.params
                });
            }
        }
    });
    groups.forEach(function (group) {
        if (group.type === types_1.StyleType.Tuples) {
            group.importedVariables = getStyleTuplesImportedVariables(group.styleTuples);
        }
    });
    return groups;
};
var getKeyframes = function (atRule) {
    var sequence = walkToArray(function (cb) { return atRule.walkRules(cb); })
        .reduce(function (accum, rule) {
        var timeSelectors = rule.selector
            .split(",")
            .map(function (selector) { return selector.trim(); })
            .map(function (selector) {
            if (/[\d.]%/.test(selector))
                return parseFloat(selector) / 100;
            if (/from/i.test(selector))
                return 0;
            if (/to/i.test(selector))
                return 1;
            throw new Error("Cannot parse keyframe time: " + selector);
        });
        var styleTuples = getStyleTuples(walkToArray(function (cb) { return rule.walkDecls(cb); }));
        var newKeyframeBlocks = timeSelectors.map(function (time) { return ({
            time: time,
            styleTuples: styleTuples
        }); });
        return accum.concat(newKeyframeBlocks);
    }, [])
        .sort(function (a, b) { return a.time - b.time; });
    var allStyleTuples = sequence.reduce(function (accum, frame) { return accum.concat(frame.styleTuples); }, []);
    var importedVariables = getStyleTuplesImportedVariables(allStyleTuples);
    var name = atRule.params;
    return { name: name, sequence: sequence, importedVariables: importedVariables };
};
exports["default"] = (function (inputCss, globals) {
    var _a = getRoot_1["default"](inputCss), root = _a.root, propTypes = _a.propTypes;
    if (globals != null)
        applyGlobals_1["default"](root, globals);
    var declarationRules = walkToArray(function (cb) { return root.walkRules(cb); }).filter(function (rule) { return !util_1.isDirectChildOfKeyframes(rule); });
    var styles = declarationRules
        .reduce(function (accum, rule) { return accum.concat(getStyleTuplesMixinsForRule(rule)); }, [])
        .filter(function (style) { return style.type === types_1.StyleType.Mixin || style.styleTuples.length !== 0; });
    var transitions = filteredStyleTuples(declarationRules, function (property) { return property in transitionAttributes; }).map(function (_a) {
        var condition = _a.condition, styleTuples = _a.styleTuples;
        return ({
            condition: condition,
            part: getTransitionAnimationParts(styleTuples, transitionAttributes),
            importedVariables: getStyleTuplesImportedVariables(styleTuples)
        });
    });
    var animations = filteredStyleTuples(declarationRules, function (property) { return property in animationAttributes; }).map(function (_a) {
        var condition = _a.condition, styleTuples = _a.styleTuples;
        return ({
            condition: condition,
            part: getTransitionAnimationParts(styleTuples, animationAttributes),
            importedVariables: getStyleTuplesImportedVariables(styleTuples)
        });
    });
    var keyframes = walkToArray(function (cb) {
        return root.walkAtRules(cb);
    })
        .filter(function (atRule) { return atRule.name === "keyframes"; })
        .map(getKeyframes);
    var exportedVariables = filteredStyleTuples(declarationRules, function (property) { return property.startsWith("--"); }).reduce(function (accum, _a) {
        var condition = _a.condition, styleTuples = _a.styleTuples;
        var declarations = styleTuples.map(function (styleTuple) {
            var importedVariables = getStyleTuplesImportedVariables([styleTuple]);
            var name = styleTuple[0].substring("--".length);
            var value = styleTuple[1];
            return { condition: condition, name: name, value: value, importedVariables: importedVariables };
        });
        return accum.concat(declarations);
    }, []);
    return {
        propTypes: propTypes,
        styles: styles,
        transitions: transitions,
        animations: animations,
        keyframes: keyframes,
        exportedVariables: exportedVariables
    };
});
